import fs from "fs-extra";
import { map } from "./map/index.mjs";
import { assembleTheme } from "./themes/assemble.mjs";
import { getThemes } from "./themes/index.mjs";
import { pipeRaw } from "./themes/raw.pipe.mjs";
import { pipeResolve } from "./themes/resolve.pipe.mjs";
import { pipeReview } from "./themes/review.pipe.mjs";
const { readJSON, outputJSON } = fs;
import { inspect } from "util";
import { optimizePaths } from "./map/paths.mjs";

export const createStyles = async (configFile) => {
  const { breakpoints, path, mediaThemes } = await readJSON(configFile);

  const themes = await getThemes({ mediaThemes, path });

  await outputJSON(`${path.destination}/log/themes.json`, themes, { spaces: 2 });

  const assembled = await Promise.all(themes.map(theme => assembleTheme(theme, path, breakpoints)));

  const combined = assembled.reduce((obj, node) => ({...obj, ...node}), {});

  // console.log(inspect(combined, { depth: 100 }));

  const raw = pipeRaw(combined, Object.keys(breakpoints));

  const resolved = pipeResolve(raw);

  const review = pipeReview(raw, mediaThemes.map(t => t.name));

  await outputJSON(`${path.destination}/styles.object.formatted.json`, resolved, { spaces: 2 });

  await outputJSON(`${path.destination}/styles.object.json`, resolved);

  await outputJSON(`${path.destination}/styles.review.json`, review, { spaces: 2 });

  const mapped = map(resolved, mediaThemes.map(t => t.name));

  await outputJSON(`${path.destination}/styles.map.formatted.json`, mapped, { spaces: 2 });
  
  await outputJSON(`${path.destination}/styles.map.json`, mapped);

  const optimizedPaths = optimizePaths(mapped);

  // console.log(optimizedPaths);
  
  await outputJSON(`${path.destination}/styles.opt.map.json`, optimizedPaths);

};
