// import fse from "fs-extra";
import { mo, ot } from "../om/index.mjs";
import { inspect } from "util";
import lodash from "lodash";
import { isBreakpointsValues } from "../om/breakpoints.mjs";
import { rx } from "../om/rx.mjs";
import { createPathsMap } from "./paths.mjs";
const { isEqual } = lodash;
// const { readJSON, outputJSON } = fse;

const isEndValue = (value) => {
  if (isBreakpointsValues(value)) return true;
  if (rx.breakpoint.value.pass.test(value)) return false;
  if (!["number", "boolean", "string", "null"].includes(ot(value))) return false;
  return true;
}


// const isPair = item => ot(item)==="array" && item.length===2 && isEndValue(item[0]) && ot(item[1])==="array" && item[1].every(e => ot(e)==="string");

// console.log(themes);

const mapPaths = (paths, themesOrder) => {

  // console.log(paths);

  const array = [];
  let i;
  for (i=0; i<paths.length; i++) {
    // console.log(paths[i]);
    const path = paths[i].split(".");
    const theme = path.shift();
    const index = themesOrder.indexOf(theme);
    array.push(index + "." + path.join("."));
    // array[index] = index + "." + path.join(".");
  }

  const reduced = [];

  // "1,2,3.a.b"
  // console.log(array);

  for (i=0; i<array.length; i++) {
    if (ot(array[i])!=="string") continue;
    const path = array[i].split(".");
    const first = path.shift();
    const rest = path.join(".");
    const existingIndex = reduced.findIndex(existingPath => {
      const existingRest = existingPath.split(".").slice(1).join(".");
      if (rest === existingRest) return true;
    });
    if (existingIndex < 0) {
      reduced.push(array[i]);
      continue;
    }
    const existingPath = reduced[existingIndex].split(".");
    const existingFirst = existingPath.shift();
    reduced[existingIndex] = existingFirst + "," + first + "." + existingPath.join(".");
  }

  // console.log(reduced);

  const splitted = reduced.map(pathString => {
    const path = pathString.split(".");
    const first = path.shift();
    return [
      first.split(",").map(v => parseInt(v)),
      [path.map(str => /^\d+$/.test(str) ? parseInt(str) : str)],
    ]
  });
  // console.log(splitted);

  // return splitted;

  const merged = [];

  for (i=0; i<splitted.length; i++) {

    const existingRecordIndex = merged.findIndex(record => {
      const themeIndexes = record[0];
      return splitted[i][0].length === themeIndexes.length && themeIndexes.every(index => splitted[i][0].includes(index))
    });

    if (existingRecordIndex < 0) {
      merged.push(splitted[i]);
      continue;
    }

    merged[existingRecordIndex] = [
      merged[existingRecordIndex][0],
      [
        ...merged[existingRecordIndex][1],
        ...splitted[i][1]
      ]
    ];

  }

  return merged;

};

export const map = (stylesObject, themesOrder) => {

  const array = [];

  mo(stylesObject, (node, path, _, index) => {
    if (!isEndValue(node)) return node;

    const pathWithIndex = index > -1 && path + "." + index;

    const existingIndex = array.findIndex((pair) => isEqual(pair[0], node));
    if (existingIndex < 0) {
      array.push([node, [pathWithIndex || path] ]);
    } else {
      array[existingIndex][1].push(pathWithIndex || path);
    }
    return node;
  });

  // console.log(array);

  array.forEach(pair => {
    // const endValue = pair[0];
    const themeSets = mapPaths(pair[1], themesOrder);
    // console.log(endValue);
    /* for (const themeSet of themeSets) {
      const [themeIndex, paths] = themeSet;
      console.log(`themes:`, themeIndex);
      console.log(`paths:`, paths);
      const pathsMap = createPathsMap(paths);
    } */
    pair[1] = themeSets;
    // console.log(inspect(pair[1], { depth: 100 }));
  });

  return array;

};
