import { isEndpoint } from "../resolve/endpoints.mjs";
import { findNext, findPrev, mo, ot } from "./index.mjs";
import { rx } from "./rx.mjs";
// import { inspect } from "util";

export const createBreakpoints = (node, breakpointNames) => {

   // console.log(`===============================`);

   // console.log(inspect(node, { depth: 100 }));

   return mo(node, (node, path) => {
      if (node.hasOwnProperty("breakpoints")) {
         // console.log(`node with breakpoints:`);
// console.log(inspect(node, { depth: 100 }));

         return create(node, breakpointNames);
      }
      return node;
   });

};

export const create = (node, breakpointNames) => {

// console.log(inspect(node, { depth: 100 }));

   const breakpointValues = breakpointNames
   .map((key) => isEndpoint(node.breakpoints[key]) && Number(node.breakpoints[key].value))
   .map((value, index, self) => {
      if (value !== undefined) return value;
      const prev = findPrev(self, index, v => v!==undefined);
      if (prev !== undefined) return prev;
      const next = findNext(self, index, v => v!==undefined);
      return next;
   });

   // console.log(`breakpoint units:`, breakpointValues);

   const res = mo(node, (node, path) => {

      if (path === "breakpoints") {
         return breakpointNames.reduce((child, key, index) => ({
            ...child,
            [key]: { value: breakpointValues[index], type: "unit" }
         }), {});
      }
      
      if (isEndpoint(node)) {
         if (/\{breakpoint\.[^\}]+\}/.test(node.value)) {
            // console.log(`expand breakpoints`);
         // expand endpoints with expressionsh {breakpoint.device} to arrays {breakpoints.xs}...{breakpoints.xxl}
            return breakpointNames.map(key => ({
               ...node,
               value: node.value.replace(/\{breakpoint\.[^\}]+\}/g, `{breakpoints.${key}}`),
            }))
         }
      }

      return node;

   });

   // console.log(res.breakpoints);

   return res;

};

export const isBreakpointsValues = node => Array.isArray(node) && node.every(
   child =>
      rx.breakpoint.value.pass.test(child)
      // ot(child) === "string" &&
      // /[^:]+:[^:]+/.test(child)
);

export const getBreakpointValue = (node, breakpoint) => {
  return mo(node, node => {
    if (isBreakpointsValues(node)) {
      console.log(`node contains array of breakpoints`);
      return (
         node.find(
            child =>
               child.indexOf(`${breakpoint}:`) === 0
         )?.split(":")[1] || node
      );   
    }
    return node;
 });
};

export const isBreakpointRef = ref => rx.breakpoint.pass.test(ref);

export const getBreakpointName = ref => [...ref.match(rx.breakpoint.extract)][1];
