import { isEndpoint } from "../resolve/endpoints.mjs";
import { mo, ot } from "./index.mjs";

const isCompositeTokenWithTypography = (node) => {
  return (
    ot(node) === "object" &&
    node.hasOwnProperty("typography") &&
    Object.entries(node).every(
      ([key, value]) =>
        key === "typography" || ["string", "number"].includes(ot(value))
    )
  );
};

export const mergeTypography = (node) =>
  mo(node, (node) => {
    if (isCompositeTokenWithTypography(node)) {
      Object.entries(node.typography).forEach(([key, value]) => {
        node[key] = value;
      });
      delete node.typography;
      return node;
    }
    return node;
  });

const types = [
  [/padding/i, "spacing"],
  [/^border$/, "fill"],
  [/^borderWidth/, "borderWidth"],
  [/width/i, "sizing"],
  [/itemSpacing/, "spacing"],
];

const correctType = value => {
  const type = types.find(type => type[0].test(value));
  if (!type) return value;
  return type[1];
};

export const expandToEndpoints = (node) =>
  mo(node, (node) => {
    if (
      isEndpoint(node) && ot(node.value) === "object" &&
      ["typography", "composition"].some((type) => node.type === type)
    ) {
      const endpointed = Object.entries(node.value).reduce((res, [key, value]) => ({
        ...res,
        [key]: { value, type: correctType(key) }
      }), {});
      return endpointed;
    }
    return node;
  });
