export const rx = {
  ref: {
    single: { pass: /^\{[^\}]+\}$/, extract: /^\{([^\}]+)\}$/ },
    pass: /\{[^\}]+\}/,
    extract: /\{([^\}]+)\}/g,
    remove: /[\{\}]/g,
  },
  expression: {
    pass: /[\+\-\/\*\(\{]/,
  },
  breakpoint: {
    pass: /\{breakpoints\.[^\}]+\}/,
    extract: /\{breakpoints\.([^\}]+)\}/,
    value: {
      pass: /^\w+:\d+/,
    }
  },
  rgba: {
    pass: /\brgba\(/,
  },
  alphaDec: {
    pass: /rgba\(\s*#[A-Fa-f0-9]{6}\s*,\s*[\d\.]+\)/,
    extract: /rgba\(\s*(#[A-Fa-f0-9]{6})\s*,\s*([\d\.]+)\)/,
  },
  px: { pass: /[\.\d]+\s*px/, extract: /([\.\d]+)\s*px/g },
  hexColor: { pass: /#[0-9A-Fa-f]{6}/, single: { pass: /^#[0-9A-Fa-f]{3,8}$/ } },

  calc: { pass: /[^\+\-\/\*\s]+\s*[\+\-\/\*]\s*[^\+\-\/\*\s]+/ },
  intOrFloat: { pass: /^[-+]?[0-9]*\.?[0-9]+$/ },
};

export const refpath = ref => {
  if (rx.ref.single.pass.test(ref)) return ref.replace(/[\{\}]/g, "");
  throw "refpath: not a ref";
};

export const splitByRefs = expression => {
  return expression.split(/({[^}]+})/,).filter(s=>s);
};
