import { ot } from "../om/index.mjs";
import { resolveRef } from "./ref.mjs";
import { splitByRefs, rx } from "../om/rx.mjs";
import reduceCSSCalc from "reduce-css-calc";
import { isCalc  } from "./endpoints.mjs";
import { getBreakpointName, isBreakpointRef } from "../om/breakpoints.mjs";
import { rgba } from "./convert.mjs";

export const resolveExpression = (endpoint, root) => {

   const expression = endpoint.value;

   // console.log(expression);

   const bp = isBreakpointRef(expression) ? getBreakpointName(expression) + ":" : "";

   const arr = splitByRefs(expression).map(part => {
      if (!rx.ref.single.pass.test(part)) return part;
      const res = resolveRef(part, root, endpoint);
      if (ot(res) !== "string" && ot(res) !== "number") {
         console.warn(`ref ${part} in expression is not a string nor a number`);
         return "";
      }

      return res;
   });

   let str = arr.join("");

   if (rx.ref.pass.test(str)) {
      console.log(`unresolved ref ${str}, try to resolve in the next pass`);
      return endpoint;
      // process.exit();
   }

   if (rx.rgba.pass.test(str)) str = rgba(str);

   // if (isCalc(endpoint)) console.log(str);

   if (isCalc(endpoint)) {
      try {
         str = reduceCSSCalc(`calc(${str})`);
      } catch (err) {
         console.log(err);
         console.log(`cannot calculate css expression ${str}`);
         process.exit();
      }
   }

   const value = bp + str;

   // const value = bp + ( isCalc(endpoint) ? reduceCSSCalc(`calc(${str})`) : str);
   
   // console.log(value);

   return {...endpoint, value };
};
