import { mo, ot, pipe } from "../om/index.mjs";
import { hasRefs, isCalc, isEndpoint, isSingleRef } from "./endpoints.mjs";
import { resolveRef } from "./ref.mjs";
import { getEndpointCss } from "./convert.mjs";
import { resolveExpression } from "./expression.mjs";
import { getBreakpointName, isBreakpointRef, isBreakpointsValues } from "../om/breakpoints.mjs";
import { shadow } from "./shadow.mjs";
import { refpath, rx } from "../om/rx.mjs";
import { getReview } from "./review.mjs";
// import lodash from "lodash";
// const { cloneDeep } = lodash;



export const resolve = (endpoint, root, review) => {

   if (!isEndpoint(endpoint)) {
      console.error("resolve: not an endpoint:", endpoint);
      process.exit();
   };

   const kind =
      isSingleRef(endpoint)
      ? "one reference to one node"
      : isCalc(endpoint)
         ? hasRefs(endpoint)
            ? "calc with refs"
            : "calc without refs"
         : hasRefs(endpoint)
            ? "refs without calc"
            : "just a value";

   // console.log(endpoint.value, ":", kind);
   /* let check;
   if (rx.rgba.pass.test(endpoint.value)) {
      console.log(`rgba:`, endpoint.value, `kind:`, kind);
      check = true;
   } */

   let resolvedValue;

   switch(kind) {
      case "one reference to one node": {
         const bp = isBreakpointRef(endpoint.value) && getBreakpointName(endpoint.value);
         const resolved = resolveRef(endpoint.value, root, endpoint);
         if (isBreakpointsValues(resolved)) return resolved;
         const value = bp ? `${bp}:${resolved}` : resolved;
         resolvedValue = getEndpointCss({...endpoint, value }).value;
         break;
      }
      case "calc with refs":
      case "refs without calc": {
         // console.log(`resolve`, endpoint.type, ":", endpoint.value);
         const resolvedEndpoint = resolveExpression(endpoint, root);
         resolvedValue = getEndpointCss(resolvedEndpoint).value;
         break;
      }
      case "just a value": {
         resolvedValue = getEndpointCss(endpoint).value;
         break;
      }
      default: {
         resolvedValue = endpoint.value;
      }
   }

   if (review) return getReview(endpoint, resolvedValue);

   return resolvedValue;

}

const resolveShadows = (node, review) => mo(node, (child) => {
   if (!isEndpoint(child)) return child;
   if (child.type === "boxShadow" && ot(child.value)==="object" && child.value.hasOwnProperty("color")) {
      const newValue = shadow(child.value);
      // console.log(newValue);
      child.value = newValue;
      return resolve(child, node, review);
   }
   return child;
});

const resolveRefs = (node, review) => mo(node, child => isEndpoint(child) ? resolve(child, node, review) : child);

export const resolveAll = (node, review) => {
   return pipe(
      node => resolveShadows(node, review),
      node => resolveRefs(node, review),
   )(node)
};
