
import lodash from "lodash";
import { fo, mo, ot } from "../om/index.mjs";
import { hasRefs, isEndpoint, isEndpointsCollection } from "./endpoints.mjs";
import { resolve } from "./index.mjs";
import { refpath, rx } from "../om/rx.mjs";
// import { isBreakpointsValues } from "../om/breakpoints.mjs";
const { get } = lodash;
import { inspect } from "util";

const findFinalNode = (ref, root) => {
   const res = get(root, refpath(ref));
   if (!res) return;
   if (isEndpoint(res) && rx.ref.single.pass.test(res.value))
      return findFinalNode(res.value, root);
   return res;
};

const findClosestNode = (ref, root, endpoint) => {

   const refNode = fo(root, node => node._id === endpoint._id);

   if (!refNode) return;

   const path = refNode.path.split(".");

   while (path.length > 0) {
      const nodeName = path.pop();
      let candidate = path.join(".");
      if (candidate) candidate += ".";
      candidate += refpath(ref);
      // console.log(`check ${candidate}`);
      const found = get(root, candidate);

      if (found && (isEndpoint(found) || isEndpointsCollection(found))) {
         // console.log(`${ref} finally found`);
         // break;
         return found;
      }
   }

   return;

};

export const resolveRef = (ref, root, endpoint, review) => {

   let node = findFinalNode(ref, root);

   if (node === undefined) {
      node = findClosestNode(ref, root, endpoint);
   }

   /* if (node === undefined) {
      console.log(`Error: can't resove ${ref}`);
      // process.exit();
   } */

   // console.log(`\n----- resolve single ref ${ref} ------`);
   // console.log(`final node:`, node);
   // console.log(`----- resolved single ref ------\n`);

   if (isEndpoint(node)) {
      if (hasRefs(node)) {
         return resolve(node, root, review);
      }
      return node.value;
   }

   if (Array.isArray(node)) return node.map(child => (
      isEndpoint(child)
      ? resolve(child, root, review)
      : child
   ));

   if (ot(node) === "object") return Object.fromEntries(Object.entries(node).map(([key, child]) => [key,
      isEndpoint(child)
      ? resolve(child, root, review)
      : child
   ]));

   // if (isBreakpointsValues(node)) return node;
   
   if (ot(node)==="string" || ot(node)==="number") return node;

   console.log(`resolveRef: can't resolve ${ref} from node:`, node);

   return ref;

}