import { mo, ot } from "../om/index.mjs";
import { rx } from "../om/rx.mjs";
import { hasRefs } from "./endpoints.mjs";

const queryKey = "🗝️";

export const getReview = (endpoint, resolvedValue) => {
  const review = {};
  if (hasRefs(endpoint)) review.expression = `{${endpoint.value.replace(rx.ref.remove, "").replace(/\./g, " ")}}`;
  if (endpoint.hasOwnProperty("description")) {
     review.comment = endpoint.description;
  };
  if (ot(resolvedValue)==="object") {
     return {
        ...resolvedValue,
        ...(review.expression && { "@ref": review.expression.replace() }),
        ...(review.comment && { "@comment": review.comment }),
     }
  }
  let str = resolvedValue.toString();
  if (review.expression) str = review.expression + " = " + str;
  if (review.comment) str += " 💬\u2009" + review.comment;
  return str;
};

const path2query = (path, themeNames) => {
   const query = path.replace(/\./g, " ");
   for (const theme of themeNames) {
     if (new RegExp(`^${theme}`).test(path)) return query.replace(theme, ``).trim(); 
   }
   return query.trim();
 };
 
export const expandPaths = (node, themeNames) => {
   return mo(node, (node, path) => {
 
     if (ot(node) === "object") {
       const query = path ? path2query(path, themeNames) : "";
       if (/undefined/.test(query)) {
         console.log("undefined:", !!path && ".");
       }
       return /\s/.test(query) && !/:/.test(query)
         ? {
             [queryKey]: query,
             ...node,
           }
         : node;
     }
     if (ot(node) === "array")
       return node.map((item, index) => {
         if (ot(item) === "object") {
           if (item[queryKey])
             return {
               [queryKey]: `${item[queryKey]} ${index}`,
               ...item,
             };
           const query =
             (path ? path2query(path, themeNames) : "") + ` ${index}`;
           return /\s/.test(query) && !/:/.test(query)
           ? {
             [queryKey]: query,
             ...item,
           } :
           item;
         }
         return item;
       });
     // Object.fromEntries(Object.entries(node).map(([key, value]) => [ (path ? path + "." : "") + key , value ]));
     return node;
   });
 }
 