import { mo, ot } from "../om/index.mjs";
import { styles } from "./styles.mjs";

const textNodeKeys = [
  "fontFamily",
  "fontWeight",
  "lineHeight",
  "fontSize",
  "letterSpacing",
  "textTransform",
  "textDecoration", // standard
  "textCase",
  "paragraphSpacing",
  "typography", // figma tokens plugin
];

const isTextNode = (node, path, index) => {
  if (Object.keys(node).some(key => textNodeKeys.includes(key))) return true;
  // zero index = container = not a text node
  return !(/\.0$|\.container$/.test(path) || index === 0);
};

export const stylize = (node, path, index) => {
  // preserve getters
  const clone = Object.create(
    Object.getPrototypeOf(node),
    Object.getOwnPropertyDescriptors(node)
  );

  styles.forEach(([key, fn]) => {
    let style;

    if (key.every(k => clone.hasOwnProperty(k))) {
      style = fn(
        key.map(k => clone[k]),
        isTextNode(clone, path, index)
      );
      key.forEach(k => {
        delete clone[k];
      });
    }

    if (style)
      Object.entries(style).forEach(([key, value]) => {
        clone[key] = value;
      });
  });

  const borderStyles = ["borderStyle", "borderTopStyle", "borderBottomStyle"];

  if (
    clone.hasOwnProperty("borderColor") &&
    !borderStyles.some(borderStyle => clone.hasOwnProperty(borderStyle))
  ) {
    ["borderWidth", "borderTopWidth", "borderBottomWidth"].forEach(
      (key, index) => {
        if (
          clone.hasOwnProperty(key) &&
          !["0", 0, "0px", "unset"].includes(clone[key])
        ) {
          clone[borderStyles[index]] = "solid";
        }
      }
    );
  }

  return clone;
};

export const stylizeAll = node =>
  mo(node, (node, path, _, index) => {
    if (ot(node) === "object") {
      return stylize(node, path, index);
    }

    return node;
  });
