
import fs from "fs-extra";
const { readJSON } = fs;

const reduceTokenSets = selectedTokenSets => Object.entries(selectedTokenSets)
.reduce((tokenSets, [key, value]) => value==="disabled" ? tokenSets : [...tokenSets, key], []);

const sortTokenSets = (tokenSets, tokenSetOrder) => {
  return tokenSets.slice(0).sort((a, b) => {
    if (tokenSetOrder.indexOf(b) < tokenSetOrder.indexOf(a)) return 1;
    if (tokenSetOrder.indexOf(b) > tokenSetOrder.indexOf(a)) return -1;
    return 0;
  });
};

export const getThemes = async ({ mediaThemes, path }) => {

  const figmaTokensThemes = await readJSON(`${path.source}/$themes.json`);
  const { tokenSetOrder } = await readJSON(`${path.source}/$metadata.json`)

  const themes = figmaTokensThemes.reduce((themes, theme) => {
    const mediaTheme = mediaThemes.find(item => item.name === theme.name);
    if (!mediaTheme) return themes;
    return [
      ...themes,
      {
        ...mediaTheme,
        tokenSets: sortTokenSets(reduceTokenSets(theme.selectedTokenSets), tokenSetOrder),
      }
    ]
  }, []);

  return themes;

};