import lodash from "lodash";
const { merge } = lodash;
import { inheritEndpoints } from "../om/om.mjs";
import { mo } from "../om/index.mjs";
// import { inspect } from "util";

export const inheritTokenSets = tokenSets =>
  tokenSets.reduce((node, tokenSet) => {
    return inheritEndpoints(tokenSet, node);
  }, {});

export const inheritComponentStates = tokenSet =>
  mo(tokenSet, (node, path) => {
    if (!path) return node;
    const key = path.split(".").pop();
    // console.log(key);
    if (
      /^[A-Z]/.test(key) &&
      node.hasOwnProperty("default") &&
      Object.keys(node).length > 1
    ) {
      // console.log(`${path} is a stateful component`);
      const states = Object.keys(node).filter(
        key => key !== "default" && /^[a-z]/.test(key)
      );

      // console.log(`states to inherit:`, states);

      const res = Object.keys(node).reduce(
        (stateful, key) =>
          states.includes(key)
            ? { ...stateful, [key]: merge({}, node.default, node[key]) }
            : { ...stateful, [key]: node[key] },
        {}
      );

      // console.log(inspect(res, { depth: 10 }));
      return res;
    }
    return node;
  });
